@@ -4,31 +4,197 @@ use 5.012;
 use strict;
 use warnings;
 use Keyword::Declare;
-#use Data::Dumper;
+use Data::Dumper;
 use Carp qw(croak);
 
+use constant { true => !!1, false => !!0 };
+
 use parent 'Exporter';
-our @EXPORT = qw( $TEST_NUMBER_OS );
+our @EXPORT = qw( $TEST_NUMBER_OS $TEST_ONLYSOME skip_these skip_next );
 
 # Docs {{{3
+
 =head1 NAME
 
 Test::OnlySome - Skip individual tests in a *.t file
 
-=head1 SYNOPSIS
+=head1 INSTALLATION
 
-TODO
+Easiest: install C<cpanminus> if you don't have it - see
+L<https://metacpan.org/pod/App::cpanminus#INSTALLATION>.  Then run
+C<cpanm Test::OnlySome>.
 
+Manually: clone or untar into a working directory.  Then, in that directory,
+
+    perl Makefile.PL
+    make
+    make test
+
+... and if all the tests pass,
+
+    make install
+
+If some of the tests fail, please check the issues and file a new one if
+no one else has reported the problem yet.
+
+=head1 USAGE
+
+    use Test::More;
     use Test::OnlySome;
 
-=head1 INTERNALS
+You can pick which tests to skip using implicit or explicit configuration.
+Explicit configuration uses a hashref:
+
+    my $opts = { skip => { 2=>true } };
+
+    os $opts ok(1, 'This will run');    # Single statement OK
+
+    os $opts {                          # Block also OK
+        ok(0, 'This will be skipped');  # Skipped since it's test 2
+    };
+
+Implicit configuration uses a hashref in the package variable C<$TEST_ONLYSOME>,
+which Test::OnlySome creates in your package when you C<use> it:
+
+    $TEST_ONLYSOME->{skip} = { 2=>true };
+    os ok(1, 'Test 1');                     # This one runs
+    os ok(0, 'Test 2 - should be skipped'); # Skipped since it's test 2
 
 =cut
 
 # }}}3
 
+# Caller-facing routines {{{1
+
+=head1 EXPORTS
+
+=head2 skip_these
+
+A convenience function to fill in C<< $hashref_options->{skip} >>.
+
+    skip_these $hashref_options, 1, 2;
+        # Skip tests 1 and 2
+    skip_these 1, 2
+        # If you are using implicit configuration
+
+=cut
+
+sub skip_these {
+    my $hrOpts = _opts($_[0]);
+    shift if $_[0] && $hrOpts == $_[0];
+    croak 'Need an options hash reference' unless ref $hrOpts eq 'HASH';
+    $hrOpts->{skip}->{$_} = true foreach(@_);
+} #skip_these()
+
+=head2 skip_next
+
+Another convenience function: Mark the next test to be skipped.
+
+=cut
+
+sub skip_next {
+    my $hrOpts = _opts($_[0]);
+    shift if $_[0] && $hrOpts == $_[0];
+    croak 'Need an options hash reference' unless ref $hrOpts eq 'HASH';
+
+    my $target = caller or croak("Couldn't find caller");
+
+    my $next_test;
+    {
+        no strict 'refs';
+        $next_test = ${ "${target}::TEST_NUMBER_OS" } or
+            croak "Couldn't get \$TEST_NUMBER_OS from $target";
+    };
+
+    $hrOpts->{skip}->{$next_test} = true;
+} #skip_next()
+
+# }}}1
+# Importer, and keyword definitions {{{1
+
+=head2 import
+
+The C<import> sub defines the keywords so that they will be exported (!).
+This is per L<Keyword::Declare>.
+
+=cut
+
+sub import {
+    my $target = caller;
+
+    # Copy symbols listed in @EXPORT first, in case @_ gets trashed later.
+    Test::OnlySome->export_to_level(1, @_);
+
+    do {
+        no strict 'refs';
+        ${ "$target" . '::TEST_NUMBER_OS' } = 1;    # tests start at 1, not 0
+        ${ "$target" . '::TEST_ONLYSOME' } = {};
+    };
+
+# `os` keyword - mark each test-calling statement this way {{{2
+
+=head2 os
+
+Keyword C<os> marks a statement that should be excuted B<o>nly B<s>ome of
+the time.  Example:
+
+    os 'main::debug' $hrOpts  ok 1,'Something';
+        # Run "ok 1,'Something'" if hashref $hrOpts indicates.
+        # Save debug information into $main::debug.
+
+Syntax:
+
+    os ['debug::variable::name'] $hashref_options <statement | block>
+
+C<$debug::variable::name> will be assigned at compilation time.
+C<$hashref_options> will be accessed at runtime.
+
+CAUTION: The given statement or block will be run in its own lexical scope,
+not in the caller's scope.
+
+=cut
+
+    keyword os(String? $debug_var, Var? $opts_name, Block|Statement $controlled) {
+        my $target = caller(2);     # Skip past Keyword::Declare's code.
+                                    # TODO make this more robust.
+
+#        # Print full stack trace
+#        my @callers;
+#        for(my $i=0; 1; ++$i) {
+#            ##       0         1          2      3            4
+#            #my ($package, $filename, $line, $subroutine, $hasargs,
+#            ##    5          6          7            8       9         10
+#            #$wantarray, $evaltext, $is_require, $hints, $bitmask, $hinthash)
+#            #= caller($i);
+#            push @callers, [caller($i)];
+#            last unless $callers[-1]->[0];
+#        }
+#        print Dumper(\@callers, "\n");
+
+        if(defined $debug_var) {
+            no strict 'refs';
+            $debug_var =~ s/^['"]|['"]$//g;   # $debug_var comes with quotes
+            ${$debug_var} = {opts_var_name => $opts_name, code => $controlled};
+            #print STDERR "# Stashed $controlled into `$debug_var`\n";
+            #print STDERR Carp::ret_backtrace(); #join "\n", caller(0);
+        }
+
+        # Get the options
+        my $hrOptsName = $opts_name || ('$' . $target . '::TEST_ONLYSOME');
+
+        croak "Need options as a scalar variable holding a hashref - got $hrOptsName"
+            unless defined $hrOptsName && substr($hrOptsName, 0, 1) eq '$';
+
+        # print STDERR "Options in $opts\n";
+        return _gen($hrOptsName, $controlled);
+    } # os() }}}2
+
+} # import()
+# }}}1
 # Implementation of keywords (macro) {{{1
 
+=head1 INTERNALS
+
 =head2 _gen
 
 This routine generates source code that, at runtime, will execute a given
@@ -66,68 +232,22 @@ sub _gen {
 
 } #_gen()
 
-# }}}1
-# Importer, and keyword definitions {{{1
+=head2 _opts
 
-=head1 EXPORTS
-
-=head2 import
-
-The C<import> sub defines the keywords so that they will be exported.
+Returns the appropriate options hashref.  Call as C<_opts $_[0]>.
 
 =cut
 
-sub import {
-    my $target = caller;
+sub _opts {
+    my $arg = shift;
+    return $arg if ref $arg eq 'HASH';
 
-    # Copy symbols listed in @EXPORT first, in case @_ gets trashed later.
-    Test::OnlySome->export_to_level(1, @_);
+    # Implicit config: find the caller's package and get $TEST_ONLYSOME
+    my $target = caller(1) or croak 'Could not find caller';
+    return do { no strict 'refs'; ${ "$target" . '::TEST_ONLYSOME' } };
 
-    do {
-        no strict 'refs';
-        ${ "$target" . '::TEST_NUMBER_OS' } = 1;    # tests start at 1, not 0
-    };
+} #_opts()
 
-# `os` keyword - mark each test-calling statement this way {{{2
-
-=head2 os
-
-Keyword C<os> marks a statement that should be excuted B<o>nly B<o>ome of
-the time.  Example:
-
-    os 'main::debug' $hrOpts  ok 1,'Something';
-        # Run "ok 1,'Something'" if hashref $hrOpts indicates.
-        # Save debug information into $main::debug.
-
-Syntax:
-
-    os ['debug::variable::name'] $hashref_options <statement | block>
-
-C<$debug::variable::name> will be assigned at compilation time.
-C<$hashref_options> will be accessed at runtime.
-
-CAUTION: The given statement or block will be run in its own lexical scope,
-not in the caller's scope.
-
-=cut
-
-    keyword os(String? $debug_var, Var $opts, Block|Statement $controlled) {
-        if(defined $debug_var) {
-            no strict 'refs';
-            $debug_var =~ s/^['"]|['"]$//g;   # $debug_var comes with quotes
-            ${$debug_var} = {opts_var_name => $opts, code => $controlled};
-            #print STDERR "# Stashed $controlled into `$debug_var`\n";
-            #print STDERR Carp::ret_backtrace(); #join "\n", caller(0);
-        }
-
-        croak "Need options as a scalar variable holding a hashref"
-            unless defined $opts && substr($opts, 0, 1) eq '$';
-
-        # print STDERR "Options in $opts\n";
-        return _gen($opts, $controlled);
-    } # os() }}}2
-
-} # import()
 # }}}1
 
 # More docs {{{3
@@ -183,11 +303,11 @@ L<https://rt.cpan.org/NoAuth/Bugs.html?Dist=Test-OnlySome>
 
 # }}}3
 
-our $VERSION = '0.000_001';
+our $VERSION = '0.000_002';
 
 =head1 VERSION
 
-Version 0.0.1
+Version 0.0.2-dev
 
 =cut
 
